@@ -17,9 +17,9 @@ import itertools
 import logging
 
 from metrique import pyclient
-from metrique.utils import utcnow
+from metrique.utils import utcnow, load
 
-logger = logging.getLogger(__name__)
+logger = logging.getLogger('metrique')
 
 
 class Rows(pyclient):
@@ -35,7 +35,8 @@ class Rows(pyclient):
     """
     name = 'csvdata_rows'
 
-    def get_objects(self, uri, _oid=None, _start=None, _end=None, **kwargs):
+    def get_objects(self, uri, _oid=None, _start=None, _end=None,
+                    load_kwargs=None, **kwargs):
         '''
         Load and transform csv data into a list of dictionaries.
 
@@ -60,7 +61,8 @@ class Rows(pyclient):
         and the result of the function will be assigned to the _start
         or _oid, respectively.
         '''
-        objects = self.load(path=uri, filetype='csv')
+        load_kwargs = load_kwargs or {}
+        objects = load(path=uri, filetype='csv', **load_kwargs)
 
         k = itertools.count(1)
         now = utcnow()
@@ -84,8 +86,3 @@ class Rows(pyclient):
             self.objects.add(obj)
 
         return super(Rows, self).get_objects(**kwargs)
-
-
-if __name__ == '__main__':
-    from metrique.argparsers import cube_cli
-    cube_cli(Rows)
