@@ -23,7 +23,7 @@ import pandas.tseries.offsets as off
 from pandas.tslib import Timestamp
 import pandas as pd
 
-from metrique.utils import dt2ts, utcnow
+from metrique.utils import dt2ts, utcnow, ts2dt
 
 logger = logging.getLogger(__name__)
 
@@ -60,13 +60,12 @@ class Result(DataFrame):
         else:
             self._lbound = self._rbound = None
             self.set_date_bounds(date)
+        # optimization:
+        self._end_isnull = self._end.isnull()
 
     def to_datetime(self, column):
         '''
-        The json serialization/deserialization process leaves dates as
-        timestamps (in s).
-
-        This function converts the column to datetimes.
+        This function converts epoch timestamps to datetimes.
 
         :param column: column to convert from current state -> datetime
         '''
@@ -87,13 +86,13 @@ class Result(DataFrame):
         if date is not None:
             split = date.split('~')
             if len(split) == 1:
-                self._lbound = Timestamp(date)
-                self._rbound = Timestamp(date)
+                self._lbound = ts2dt(date)
+                self._rbound = ts2dt(date)
             elif len(split) == 2:
                 if split[0] != '':
-                    self._lbound = Timestamp(split[0])
+                    self._lbound = ts2dt(split[0])
                 if split[1] != '':
-                    self._rbound = Timestamp(split[1])
+                    self._rbound = ts2dt(split[1])
             else:
                 raise Exception('Date %s is not in the correct format' % date)
 
@@ -118,7 +117,7 @@ class Result(DataFrame):
             raise ValueError('Date %s is not in the queried range.' % date)
         date = Timestamp(date)
         after_start = self._start <= date
-        before_end = (self._end > date) | self._end.isnull()
+        before_end = (self._end > date) | self._end_isnull
         if only_count:
             return np.sum(before_end & after_start)
         else:
@@ -190,7 +189,7 @@ class Result(DataFrame):
         end = Timestamp(end or max(Timestamp(self._end.max()),
                                    self._start.max()))
         # FIXME: end != end ?
-        end = utcnow(as_datetime=True) if repr(end) == 'NaT' else end
+        end = utcnow() if repr(end) == 'NaT' else end
         start = start if self.check_in_bounds(start) else self._lbound
         end = end if self.check_in_bounds(end) else self._rbound
 
@@ -260,7 +259,7 @@ class Result(DataFrame):
         Leaves only versions of those objects that has some version with
         `_end == None` or with `_end > right cutoff`.
         '''
-        mask = self._end.isnull()
+        mask = self._end_isnull
         if self._rbound is not None:
             mask = mask | (self._end > self._rbound)
         oids = set(self[mask]._oid.tolist())
@@ -317,7 +316,7 @@ class Result(DataFrame):
             last[col_name] = age - timedelta(microseconds=age.microseconds)
             return last
 
-        cut_ts = self._rbound or utcnow(as_datetime=True)
+        cut_ts = self._rbound or utcnow()
         res = pd.concat([prep(df) for _, df in self.groupby(self._oid)])
         return res
 
@@ -396,3 +395,30 @@ class Result(DataFrame):
         :param function: func that takes a DataFrame and returns a DataFrame
         '''
         return pd.concat([function(df) for _, df in self.groupby(self._oid)])
+
+    def has(self, field, val):
+        return self[field].apply(lambda vals: val in (vals or []))
+
+    @filtered
+    def fhas(self, field, val):
+        return self[self.has(field, val)]
+
+    def isin(self, field, vals):
+        return self[field].apply(lambda x:
+                                 any([e in vals for e in x])
+                                 if isinstance(x, list)
+                                 else x in vals)
+
+    @filtered
+    def fisin(self, field, vals):
+        return self[self.isin(field, vals)]
+
+    @filtered
+    def fnotin(self, field, vals):
+        return self[~self.isin(field, vals)]
+
+    def isempty(self, field):
+        return ~self[field].astype(bool)
+
+    def notempty(self, field):
+        return self[field].astype(bool)
